@@ -20,5 +20,7 @@
 require "y2storage/callbacks/initialize"
 require "y2storage/callbacks/activate"
 require "y2storage/callbacks/probe"
+require "y2storage/callbacks/user_probe"
+require "y2storage/callbacks/yast_probe"
 require "y2storage/callbacks/commit"
 require "y2storage/callbacks/check"
