@@ -5,6 +5,8 @@ require "shellwords"
 require "packager/product_patterns"
 require "y2packager/resolvable"
 require "y2packager/repository"
+require "uri"
+require "yast2/rel_url"
 
 # Yast namespace
 module Yast
@@ -26,10 +28,6 @@ module Yast
       # SLE12 HA GEO is now included in SLE15 HA
       "sle-ha-geo"                        => ["sle-ha"],
       "SUSE_SLES_SAP"                     => ["SLES_SAP"],
-      # SLES-12 with HPC module can be replaced by SLE_HPC-15
-      "SLES"                              => ["SLE_HPC"],
-      # this is an internal product so far...
-      "SLE-HPC"                           => ["SLE_HPC"],
       # SMT is now integrated into the base SLES
       "sle-smt"                           => ["SLES"],
       # Live patching is a module now (bsc#1074154)
@@ -41,10 +39,38 @@ module Yast
       "sle-sdk"                           => ["sle-module-development-tools"],
       # openSUSE => SLES migration
       "openSUSE"                          => ["SLES"],
+      # openSUSE 15.3+ => SLES migration
+      "Leap"                              => ["SLES"],
       # the IBM tools have been renamed in SLE12->SLE15 upgrade
       "ibm-dlpar-utils"                   => ["ibm-power-tools"]
+
+      # NOTE: if you change anything here then check the MAPPING value
+      # in the src/lib/y2packager/product_upgrade.rb file, maybe it needs an update as well...
     }.freeze
 
+    private_constant :DEFAULT_PRODUCT_RENAMES
+
+    attr_accessor :new_renames
+
+    def default_product_renames
+      # special handling for the HPC product
+      product_mapping = if new_renames
+        {
+          # in SLE15-SP6+ SLE_HPC is replaced by SLES + HPC module
+          "SLE-HPC" => ["SLES"],
+          "SLE_HPC" => ["SLES"]
+        }
+      else
+        {
+          # in SLE15 SP5 (or older) the SLES + HPC module is upgraded to SLE_HPC
+          "SLES"    => ["SLE_HPC"],
+          "SLE-HPC" => ["SLE_HPC"]
+        }
+      end
+
+      product_mapping.merge!(DEFAULT_PRODUCT_RENAMES)
+    end
+
     # @return [Hash] Product renames added externally through the #add_rename method
     attr_accessor :external_product_renames
     private :external_product_renames, :external_product_renames=
@@ -221,14 +247,15 @@ module Yast
 
       optional = true if optional.nil?
 
-      if sod == "signed"
+      case sod
+      when "signed"
         provided_file = Pkg.SourceProvideSignedFile(
           src_id,
           media,
           filename,
           optional
         )
-      elsif sod == "digested"
+      when "digested"
         provided_file = Pkg.SourceProvideDigestedFile(
           src_id,
           media,
@@ -251,7 +278,7 @@ module Yast
         cached_file = Builtins.sformat("%1%2", @filecachedir, @filecachecounter)
 
         cmd = "/usr/bin/mkdir -p #{@filecachedir.shellescape}; " \
-          "/usr/bin/cp #{provided_file.shellescape} #{cached_file.shellescape}"
+              "/usr/bin/cp #{provided_file.shellescape} #{cached_file.shellescape}"
         cmd_run = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
 
         # Unable to cache a file, the original file will be returned
@@ -382,23 +409,49 @@ module Yast
     end
 
     # Returns an absolute URL from base + relative url.
-    # Relative URL needs to start with 'reulrl://' othewise
-    # it is not considered being relative and it's returned
-    # as it is (just the relative_url parameter).
+    #
+    # Relative URL needs to start with 'relurl://' or 'repo:/', otherwise it
+    # is not considered being relative and it's returned as it is (just the
+    # relative_url parameter).
+    #
+    # 'repo' URLs are resolved relative to the installation medium, not base_url.
     #
     # @param [String] base_url
-    # @param [String] url URL relative to the base
+    # @param [String] url URL relative to base_url resp. installation medium
     #
     # @example
     #   AddOnProduct::GetAbsoluteURL (
     #     "http://www.example.org/some%20dir/another%20dir",
     #     "relurl://../AnotherProduct/"
     #   ) -> "http://www.example.org/some%20dir/AnotherProduct/"
+    #
     #   AddOnProduct::GetAbsoluteURL (
-    #     "username:password@ftp://www.example.org/dir/",
+    #     "ftp://username:password@www.example.org/dir/",
     #     "relurl://./Product_CD1/"
-    #   ) -> "username:password@ftp://www.example.org/dir/Product_CD1/"
+    #   ) -> "ftp://username:password@www.example.org/dir/Product_CD1/"
+    #
+    #   with /etc/install.inf containing the line 'ZyppRepoURL: hd:/?device=/dev/sda'
+    #   AddOnProduct::GetAbsoluteURL (
+    #     "ftp://username:password@www.example.org/dir/",
+    #     "repo:/Product_CD1"
+    #   ) -> "hd:/Product_CD1?device=/dev/sda"
     def GetAbsoluteURL(base_url, url)
+      url_parsed = URI(url)
+
+      if url_parsed.scheme == "repo"
+        base_url = InstURL.installInf2Url("")
+        if base_url.empty?
+          log.error "no ZyppRepoURL in /etc/install.inf"
+        else
+          rel_url = "relurl:" + url_parsed.path
+          url = Yast2::RelURL.from_installation_repository(rel_url).absolute_url.to_s
+          log.info("base url + relative path: #{URL.HidePassword(base_url)} + #{url_parsed.path}")
+          log.info("absolute url: #{URL.HidePassword(url)}")
+        end
+
+        return url
+      end
+
       if !Builtins.regexpmatch(url, "^relurl://")
         Builtins.y2debug("Not a relative URL: %1", URL.HidePassword(url))
         return url
@@ -516,10 +569,10 @@ module Yast
         path(".target.bash_output"),
         Builtins.sformat(
           "\n" \
-            "/usr/bin/mkdir %1;\n" \
-            "cd %1;\n" \
-            "/bin/tar -xvf %2;\n" \
-            "/sbin/adddir %1 /;\n",
+          "/usr/bin/mkdir %1;\n" \
+          "cd %1;\n" \
+          "/bin/tar -xvf %2;\n" \
+          "/sbin/adddir %1 /;\n",
           tmpdir.shellescape,
           filename.shellescape
         )
@@ -622,20 +675,22 @@ module Yast
       binaries ||= y2update_path(src_id)
 
       # File /y2update.tgz exists
-      if !binaries.nil?
+      if binaries.nil?
+        Builtins.y2milestone("File /y2update.tgz not provided")
+      else
         # Try to extract files from the archive
         out = Convert.to_map(
           SCR.Execute(
             path(".target.bash_output"),
             Builtins.sformat(
               "\n" \
-                "/usr/bin/test -d /y2update && /usr/bin/rm -rf /y2update;\n" \
-                "/bin/mkdir -p /y2update/all;\n" \
-                "cd /y2update/all;\n" \
-                "/bin/tar -xvf %1;\n" \
-                "cd /y2update;\n" \
-                "/usr/bin/ln -s all/usr/share/YaST2/* .;\n" \
-                "/usr/bin/ln -s all/usr/lib/YaST2/* .;\n",
+              "/usr/bin/test -d /y2update && /usr/bin/rm -rf /y2update;\n" \
+              "/bin/mkdir -p /y2update/all;\n" \
+              "cd /y2update/all;\n" \
+              "/bin/tar -xvf %1;\n" \
+              "cd /y2update;\n" \
+              "/usr/bin/ln -s all/usr/share/YaST2/* .;\n" \
+              "/usr/bin/ln -s all/usr/lib/YaST2/* .;\n",
               binaries.shellescape
             )
           )
@@ -653,8 +708,6 @@ module Yast
           # bugzilla #239055
           RereadAllSCRAgents()
         end
-      else
-        Builtins.y2milestone("File /y2update.tgz not provided")
       end
 
       true
@@ -822,7 +875,7 @@ module Yast
 
           if !package_installed
             Report.Error(_("Package '%s' is not installed.\n" \
-              "The add-on product cannot be registered.") % "yast2-registration")
+                           "The add-on product cannot be registered.") % "yast2-registration")
             return nil
           end
         end
@@ -1138,7 +1191,7 @@ module Yast
             Label(
               _(
                 "The installation repository also contains the listed additional repositories.\n" \
-                  "Select the ones you want to use.\n"
+                "Select the ones you want to use.\n"
               )
             )
           ),
@@ -1250,14 +1303,14 @@ module Yast
             "Selecting product '%1' for installation",
             one_prod
           )
-          if !Pkg.ResolvableInstall(one_prod, :product)
+          if Pkg.ResolvableInstall(one_prod, :product)
+            @selected_installation_products << one_prod
+          else
             Report.Error(
               # TRANSLATORS: Product cannot be set for installation.
               # %1 is the product name.
               Builtins.sformat(_("Product %1 not found on media."), one_prod)
             )
-          else
-            @selected_installation_products << one_prod
           end
         end
 
@@ -1577,7 +1630,7 @@ module Yast
       end
 
       # set repository alias to product name or alias if specified
-      if !name.nil? && name != "" || !alias_name.nil? && alias_name != ""
+      if (!name.nil? && name != "") || (!alias_name.nil? && alias_name != "")
         url_p = URL.Parse(url)
         params = URL.MakeMapFromParams(Ops.get_string(url_p, "query", ""))
         new_alias = ""
@@ -1677,7 +1730,10 @@ module Yast
           configuration_from_disk
         )
 
-        if !configuration_from_disk.nil?
+        if configuration_from_disk.nil?
+          Builtins.y2error("Reading %1 file returned nil result!", tmp_filename)
+          false
+        else
           Import(configuration_from_disk)
           if already_in_configuration != [] && !already_in_configuration.nil?
             @add_on_products = Convert.convert(
@@ -1686,14 +1742,11 @@ module Yast
               to:   "list <map <string, any>>"
             )
           end
-          return true
-        else
-          Builtins.y2error("Reading %1 file returned nil result!", tmp_filename)
-          return false
+          true
         end
       else
         Builtins.y2warning("File %1 doesn't exists, skipping...", tmp_filename)
-        return true
+        true
       end
     end
 
@@ -1981,7 +2034,7 @@ module Yast
 
       # handle the product renames, if a renamed product was installed
       # replace it with the new product so the new product is preselected
-      DEFAULT_PRODUCT_RENAMES.each do |k, v|
+      default_product_renames.each do |k, v|
         next unless installed_addons.include?(k)
 
         installed_addons.delete(k)
@@ -2004,7 +2057,7 @@ module Yast
     publish variable: :low_memory_already_reported, type: "boolean"
     publish variable: :skip_add_ons, type: "boolean"
     publish function: :GetCachedFileFromSource,
-            type:     "string (integer, integer, string, string, boolean)"
+      type:     "string (integer, integer, string, string, boolean)"
     publish function: :AddOnMode, type: "string (integer)"
     publish function: :SetBaseProductURL, type: "void (string)"
     publish function: :GetBaseProductURL, type: "string ()"
@@ -2049,7 +2102,7 @@ module Yast
     # @see #renamed_at?
     def renamed_externally?(old_name, new_name)
       log.info "Search #{old_name} -> #{new_name} rename in added renames map: " \
-        "#{external_product_renames.inspect}"
+               "#{external_product_renames.inspect}"
       renamed_at?(external_product_renames, old_name, new_name)
     end
 
@@ -2066,18 +2119,18 @@ module Yast
     def renamed_by_libzypp?(old_name, new_name)
       renames = product_renames_from_libzypp
       log.info "Search #{old_name} -> #{new_name} rename in libzypp: " \
-        "#{renames.inspect}"
+               "#{renames.inspect}"
       renamed_at?(renames, old_name, new_name)
     end
 
     # Determine whether a product rename is included in the fallback map
     #
-    # @see DEFAULT_PRODUCT_RENAMES
+    # @see default_product_renames
     # @see #renamed_at?
     def renamed_by_default?(old_name, new_name)
       log.info "Search #{old_name} -> #{new_name} rename in default map: " \
-        "#{DEFAULT_PRODUCT_RENAMES.inspect}"
-      renamed_at?(DEFAULT_PRODUCT_RENAMES, old_name, new_name)
+               "#{default_product_renames.inspect}"
+      renamed_at?(default_product_renames, old_name, new_name)
     end
 
     # Determine whether a product rename is present on a given hash
@@ -2116,7 +2169,7 @@ module Yast
     # Regular expresion to extract the product name. It supports two different
     # formats: "product:NAME" and "product(NAME)"
     # @see product_name_from_dep
-    PRODUCTS_REGEXP = /\Aproduct(?::|\()([^\)\b]+)/.freeze
+    PRODUCTS_REGEXP = /\Aproduct(?::|\()([^)\b]+)/.freeze
 
     # Extracts product's name from dependency
     #
@@ -2196,7 +2249,7 @@ module Yast
           return repo_id if found_product["label"] == prodname
 
           log.info("Removing repo #{repo_id}: Add-on found #{found_product["label"]}," \
-            "expected: #{prodname}")
+                   "expected: #{prodname}")
           Pkg.SourceDelete(repo_id)
         end
 
